==> cv-react-library-0.1.16.tgz: package/src/components/native/DropDown/DropDown.js <==
import React, { Component, Fragment } from 'react';
import PropTypes from 'prop-types';
import { ActivityIndicator, Dimensions, TouchableOpacity, View } from 'react-native';
import Modal from '../Modal/Modal';
import TextLabel from '../TextLabel/TextLabel';
import MenuItem from '../MenuItem/MenuItem';

// Styles
import getStyles from './DropDown.styles';

/**
 * A picker for the selection of a menu item.
 */
class DropDown extends Component {
    static propTypes = {
        /** Styles for this component */
        contextStyles: PropTypes.shape({
            /** Styles for the container surrounding the text and icon */
            container: PropTypes.object,

            /** Styles for the icon */
            icon: PropTypes.object,

            /** Styles for the menu container */
            menu: PropTypes.object,

            /** Styles for the menu item icon */
            menuIcon: PropTypes.object,

            /** Styles for the menu item container */
            menuItem: PropTypes.object,

            /** Styles for the menu overlay */
            menuOverlay: PropTypes.object,

            /** Styles for the menu item text */
            menuText: PropTypes.object,

            /** Styles for the value text */
            text: PropTypes.object,
        }),

        /** Prevent press events from showing the menu */
        disabled: PropTypes.bool,

        /** Empty item data */
        emptyItem: PropTypes.shape({
            data: PropTypes.any,
            icon: PropTypes.string,
            text: PropTypes.string,
        }),

        /** An array of data with which to render into drop down menu items */
        items: PropTypes.arrayOf(PropTypes.shape({
            data: PropTypes.any,
            icon: PropTypes.string,
            text: PropTypes.string,
        })),

        /**
         * Return a method that resolves to 'items' prop
         * @return {Array<{data: *, icon: String, text: String}>}
         */
        itemsResolver: PropTypes.func,

        /**
         * Called when the selection changes
         * @param {{data: *, icon: String, text: String}} item - The pressed option data
         * @param {SyntheticEvent} event - The native press event
         */
        onChange: PropTypes.func,

        /** Close menu on overlay press */
        overlayClose: PropTypes.bool,

        /** Enables a blank/empty menu option as the first item */
        showEmptyItem: PropTypes.bool,

        /** The item text to display as the currently selected value */
        text: PropTypes.string,

        /** Id used for testing */
        testID: PropTypes.string,

        /** Menu visibility */
        visible: PropTypes.bool,
    };

    static defaultProps = {
        contextStyles: {},
        emptyItem: {
            data: { value: '' },
            icon: null,
            text: '(empty)',
        },
        overlayClose: true,
        text: '',
    };

    constructor(props) {
        super(props);
        const {
            text,
            visible,
        } = props;
        this.state = {
            disableSelection: false,
            items: null,
            loading: false,
            selectedValue: text,
            visible,
        };
    }

    render() {
        const {
            disabled,
            emptyItem,
            contextStyles,
            overlayClose,
            showEmptyItem,
            testID,
        } = this.props;
        const {
            disableSelection,
            items,
            loading,
            selectedValue,
            visible,
        } = this.state;

        const styles = getStyles(contextStyles);

        // Generate touchable props
        const touchableProps = {
            activeOpacity: 0.5,
        };
        if (!disabled) { touchableProps.onPress = this.showMenu; }
        if (testID) { touchableProps.testID = `${testID}__touchable`; }

        // Generate container props
        const containerProps = {
            style: styles.container,
        };
        if (disabled) { containerProps.style = styles.disabledContainer; }
        if (testID) { containerProps.testID = `${testID}__container`; }

        // Generate input text props
        const textProps = {
            contextStyles: {
                text: styles.text,
            },
        };
        if (testID) { textProps.testID = `${testID}__text`; }

        // Generate icon props
        const iconProps = {
            contextStyles: {
                text: styles.icon,
            },
        };
        if (testID) { iconProps.testID = `${testID}__icon`; }

        // Generate modal props
        const modalProps = {
            contextStyles: {
                modal: styles.menu,
                overlay: styles.menuOverlay,
            },
        };
        if (loading) {
            modalProps.visible = true;
            modalProps.children = <ActivityIndicator color="#000" />;
            modalProps.key = 'loading';
            modalProps.contextStyles.modal = {
                ...styles.loadingContainer,
            };
            if (testID) { modalProps.testID = `${testID}__loading`; }
        }
        else if (visible) {
            modalProps.visible = true;
            modalProps.key = 'menu';
            if (overlayClose) {
                modalProps.overlayClose = overlayClose && !disableSelection;
                modalProps.onClose = this.hideMenu;
            }
            if (testID) { modalProps.testID = `${testID}__modal`; }

            // Build menu items
            let menuItems = [];
            if (Array.isArray(items)) {
                menuItems = items.map((item) => (this.renderMenuItem(item, styles)));
            }
            modalProps.children = menuItems;

            // Add 'empty' item if desired
            if (showEmptyItem) {
                const emptyOption = this.renderMenuItem(emptyItem, styles);
                modalProps.children = [ emptyOption ].concat(menuItems);
            }
        }

        return (
            <Fragment>
                <TouchableOpacity { ...touchableProps }>
                    <View { ...containerProps }>
                        <TextLabel { ...textProps }>
                            { selectedValue }
                        </TextLabel>
                        <TextLabel { ...iconProps }>&#9660;</TextLabel>
                    </View>
                </TouchableOpacity>
                { (visible || loading) && <Modal { ...modalProps } /> }
            </Fragment>
        );
    }

    componentDidUpdate(prevProps) {
        const { visible: wasVisible } = prevProps;
        const { visible: isVisible } = this.props;
        if (isVisible !== wasVisible) {
            this.reset({ visible: isVisible });
        }
    }

    componentDidMount() {
        Dimensions.addEventListener('change', this.handleOrientationChange);
    }

    componentWillUnmount() {
        Dimensions.removeEventListener('change', this.handleOrientationChange);
    }

    /**
     * Generate individual menu item
     * @param {{ data: Object, icon: String, text: String }} item
     * @param {Object} styles
     * @return {React.node}
     */
    renderMenuItem = (item, styles) => {
        const {
            selectedValue,
            disableSelection,
        } = this.state;
        const menuItemProps = {
            contextStyles: {
                container: {
                    minWidth: 200,
                    ...styles.menuItem,
                },
                icon: styles.menuIcon,
                text: styles.menuText,
            },
            disabled: disableSelection,
            key: item.text || item.icon || JSON.stringify(item.data),
            onPress: (event) => { this.handleOnChange(item, event); },
            selected: selectedValue === item.text,
        };
        if (item.text) { menuItemProps.text = item.text; }
        if (item.icon) { menuItemProps.iconName = item.icon; }

        return <MenuItem { ...menuItemProps } />;
    };

    /**
     * Called on press of the menu item
     * @param {{ data: Object, icon: String, text: String }} item
     * @param {SyntheticEvent} event - The native press event
     */
    handleOnChange = (item, event) => {
        const { onChange } = this.props;
        this.setState({
            selectedValue: item.text,
            disableSelection: true,
        }, () => {
            setTimeout(() => {
                this.setState({
                    visible: false,
                    disableSelection: false,
                }, () => {
                    if (onChange) {
                        onChange(item, event);
                    }
                });
            }, 250);
        });
    };

    /**
     * Hide the menu of options
     */
    hideMenu = () => {
        this.setState({
            visible: false,
        });
    };

    /**
     * Show the menu of options
     */
    showMenu = () => {
        const {
            items,
            itemsResolver: getData,
        } = this.props;

        if (getData) {
            this.setState({
                loading: true,
            }, () => {
                Promise.resolve(getData())
                    .then((data) => {
                        this.setState({
                            items: data,
                            loading: false,
                            visible: true,
                        });
                    });
            });
        }
        else if (Array.isArray(items)) {
            this.setState({
                items,
                visible: true,
            });
        }
        else {
            this.setState({
                visible: true,
            });
        }
    };

    /**
     * Handle orientation changes
     */
    handleOrientationChange = () => {
        this.hideMenu();
    };

    /**
     * Handles state updates after componentDidUpdate
     * @param {Object} newState
     */
    reset = (newState) => {
        this.setState(newState);
    };
}

export default DropDown;

==> cv-react-library-0.1.29.tgz: package/src/components/native/DropDown/DropDown.js <==
import React, { Component, Fragment } from 'react';
import PropTypes from 'prop-types';
import { ActivityIndicator, Dimensions, TouchableOpacity } from 'react-native';
import Modal from '../Modal/Modal';
import TextLabel from '../TextLabel/TextLabel';
import MenuItem from '../MenuItem/MenuItem';

// Styles
import getStyles from './DropDown.styles';

/**
 * A picker for the selection of a menu item.
 */
class DropDown extends Component {
    static propTypes = {
        /** Styles for this component */
        contextStyles: PropTypes.shape({
            /** Styles for the container surrounding the text and icon */
            container: PropTypes.object,

            /** Styles for the icon */
            icon: PropTypes.object,

            /** Styles for the menu container */
            menu: PropTypes.object,

            /** Styles for the menu item icon */
            menuIcon: PropTypes.object,

            /** Styles for the menu item container */
            menuItem: PropTypes.object,

            /** Styles for the menu overlay */
            menuOverlay: PropTypes.object,

            /** Styles for the menu item text */
            menuText: PropTypes.object,

            /** Styles for the value text */
            text: PropTypes.object,
        }),

        /** Prevent press events from showing the menu */
        disabled: PropTypes.bool,

        /** Empty item data */
        emptyItem: PropTypes.shape({
            data: PropTypes.any,
            icon: PropTypes.string,
            text: PropTypes.string,
        }),

        /** An array of data with which to render into drop down menu items */
        items: PropTypes.arrayOf(PropTypes.shape({
            data: PropTypes.any,
            icon: PropTypes.string,
            text: PropTypes.string,
        })),

        /**
         * Return a method that resolves to 'items' prop
         * @return {Array<{data: *, icon: String, text: String}>}
         */
        itemsResolver: PropTypes.func,

        /**
         * Called when the selection changes
         * @param {{data: *, icon: String, text: String}} item - The pressed option data
         * @param {SyntheticEvent} event - The native press event
         */
        onChange: PropTypes.func,

        /** Close menu on overlay press */
        overlayClose: PropTypes.bool,

        /** Enables a blank/empty menu option as the first item */
        showEmptyItem: PropTypes.bool,

        /** The item text to display as the currently selected value */
        text: PropTypes.string,

        /** Id used for testing */
        testID: PropTypes.string,

        /** Menu visibility */
        visible: PropTypes.bool,
    };

    static defaultProps = {
        contextStyles: {},
        emptyItem: {
            data: { value: '' },
            icon: null,
            text: '(empty)',
        },
        overlayClose: true,
        text: '',
    };

    constructor(props) {
        super(props);
        const {
            text,
            visible,
        } = props;
        this.state = {
            disableSelection: false,
            items: null,
            loading: false,
            selectedValue: text,
            visible,
        };
    }

    render() {
        const {
            disabled,
            emptyItem,
            contextStyles,
            overlayClose,
            showEmptyItem,
            testID,
        } = this.props;
        const {
            disableSelection,
            items,
            loading,
            selectedValue,
            visible,
        } = this.state;

        const styles = getStyles(contextStyles);

        // Generate container props
        const containerProps = {
            activeOpacity: 0.5,
            style: styles.container,
        };
        if (!disabled) { containerProps.onPress = this.showMenu; }
        if (disabled) { containerProps.style = styles.disabledContainer; }
        if (testID) { containerProps.testID = `${testID}__container`; }

        // Generate input text props
        const textProps = {
            contextStyles: {
                text: styles.text,
            },
        };
        if (testID) { textProps.testID = `${testID}__text`; }

        // Generate icon props
        const iconProps = {
            contextStyles: {
                text: styles.icon,
            },
        };
        if (testID) { iconProps.testID = `${testID}__icon`; }

        // Generate modal props
        const modalProps = {
            contextStyles: {
                modal: styles.menu,
                overlay: styles.menuOverlay,
            },
        };
        if (loading) {
            modalProps.visible = true;
            modalProps.children = <ActivityIndicator color="#000" />;
            modalProps.key = 'loading';
            modalProps.contextStyles.modal = {
                ...styles.loadingContainer,
            };
            if (testID) { modalProps.testID = `${testID}__loading`; }
        }
        else if (visible) {
            modalProps.visible = true;
            modalProps.key = 'menu';
            if (overlayClose) {
                modalProps.overlayClose = overlayClose && !disableSelection;
                modalProps.onClose = this.hideMenu;
            }
            if (testID) { modalProps.testID = `${testID}__modal`; }

            // Build menu items
            let menuItems = [];
            if (Array.isArray(items)) {
                menuItems = items.map((item) => (this.renderMenuItem(item, styles)));
            }
            modalProps.children = menuItems;

            // Add 'empty' item if desired
            if (showEmptyItem) {
                const emptyOption = this.renderMenuItem(emptyItem, styles);
                modalProps.children = [ emptyOption ].concat(menuItems);
            }
        }

        return (
            <Fragment>
                <TouchableOpacity { ...containerProps }>
                    <TextLabel { ...textProps }>
                        { selectedValue }
                    </TextLabel>
                    <TextLabel { ...iconProps }>&#9660;</TextLabel>
                </TouchableOpacity>
                { (visible || loading) && <Modal { ...modalProps } /> }
            </Fragment>
        );
    }

    componentDidUpdate(prevProps) {
        const {
            visible: wasVisible,
            text: oldText,
        } = prevProps;
        const {
            visible: isVisible,
            text: newText,
        } = this.props;
        if (isVisible !== wasVisible) {
            this.reset({ visible: isVisible });
        }
        if (oldText !== newText) {
            this.reset({ selectedValue: newText });
        }
    }

    componentDidMount() {
        Dimensions.addEventListener('change', this.handleOrientationChange);
    }

    componentWillUnmount() {
        Dimensions.removeEventListener('change', this.handleOrientationChange);
    }

    /**
     * Generate individual menu item
     * @param {{ data: Object, icon: String, text: String }} item
     * @param {Object} styles
     * @return {React.node}
     */
    renderMenuItem = (item, styles) => {
        const {
            selectedValue,
            disableSelection,
        } = this.state;
        const menuItemProps = {
            contextStyles: {
                container: {
                    minWidth: 200,
                    ...styles.menuItem,
                },
                icon: styles.menuIcon,
                text: styles.menuText,
            },
            disabled: disableSelection,
            key: item.text || item.icon || JSON.stringify(item.data),
            onPress: (event) => { this.handleOnChange(item, event); },
            selected: selectedValue === item.text,
        };
        if (item.text) { menuItemProps.text = item.text; }
        if (item.icon) { menuItemProps.iconName = item.icon; }

        return <MenuItem { ...menuItemProps } />;
    };

    /**
     * Called on press of the menu item
     * @param {{ data: Object, icon: String, text: String }} item
     * @param {SyntheticEvent} event - The native press event
     */
    handleOnChange = (item, event) => {
        const { onChange } = this.props;
        this.setState({
            selectedValue: item.text,
            disableSelection: true,
        }, () => {
            setTimeout(() => {
                this.setState({
                    visible: false,
                    disableSelection: false,
                }, () => {
                    if (onChange) {
                        onChange(item, event);
                    }
                });
            }, 250);
        });
    };

    /**
     * Hide the menu of options
     */
    hideMenu = () => {
        this.setState({
            visible: false,
        });
    };

    /**
     * Show the menu of options
     */
    showMenu = () => {
        const {
            items,
            itemsResolver: getData,
        } = this.props;

        if (getData) {
            this.setState({
                loading: true,
            }, () => {
                Promise.resolve(getData())
                    .then((data) => {
                        this.setState({
                            items: data,
                            loading: false,
                            visible: true,
                        });
                    });
            });
        }
        else if (Array.isArray(items)) {
            this.setState({
                items,
                visible: true,
            });
        }
        else {
            this.setState({
                visible: true,
            });
        }
    };

    /**
     * Handle orientation changes
     */
    handleOrientationChange = () => {
        this.hideMenu();
    };

    /**
     * Handles state updates after componentDidUpdate
     * @param {Object} newState
     */
    reset = (newState) => {
        this.setState(newState);
    };
}

export default DropDown;
